#!/usr/bin/env python
# dungeon_map_maker.py
# by Keith Reinig and Rob Antonishen
# Version 1.8 (20200403)

# Description

# Make a more "artistic" dungeon map using monochrome template
# This is a python port of Rob A's script-fu version from cartographers guild
# All credit for the methods is his

# This port does not include the batch mode

# http://cartographersguild.com

# Changes
# 1.7 - original port.  Kept same version number for clarity
#       No Batch Mode
#       HEX Grid Removed as I couldn't get the hex script to work with
#       GIMP 2.10
#       Combined the Stroke Style and Bevel Stroke items in an attempt
#           to shorten the input dialog somewhat
#       Remove the Distortion settings.  Couldn't figure out what they
#           were doing anyway
# 1.8 - Fixed an issue with the blur on the wall shadows and wall bevel


# License:

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# The GNU Public License is available at
# http://www.gnu.org/copyleft/gpl.html

import math
import random
from gimpfu import *

def dungeon_map_maker(image, inLayer, inGridSize, inSelectColor, inSelectThresh, inSelectWhat, inLightAngle,
                      inFloorFill, inFloorPattern, inNoiseOverlay,  inGrid, inGridColor, inWallMode, inWallPattern, inWallStyle,
                      inWallFade, inStrokeStyle, inStrokeSize, inStrokePos, inStrokeColor, inStrokePattern):
    
    
    
    width = 0
    height = 0
    varLayerName = ""
    randSeed = 0
    layerFloor = 0
    layerFloorOverlay = 0
    layerWall = 0
    maskWall = 0
    layerGrid = 0
    layerGridBump = 0
    layerWallFade = 0
    layerWallStyle = 0
    layerWallShadows = 0
    selInside = 0
    layerTemp1 = 0
    layerTemp2 = 0
    layerTemp3 = 0
    handler = pdb.gimp_message_get_handler()
    origLayer = 0
  
  
  # it begins here
        
    height = pdb.gimp_image_height(image)
    width = pdb.gimp_image_width(image)
    varLayerName = inLayer.name;
    
    pdb.gimp_context_push()
    pdb.gimp_progress_init("Working...", None);
    pdb.gimp_progress_pulse();
    pdb.gimp_image_undo_group_start(image)
    
    interpolation = pdb.gimp_context_get_interpolation()
    transform = pdb.gimp_context_get_transform_resize()
            
    
           
    pdb.gimp_message("Getting Started: " + image.name)
    pdb.gimp_message("Clearing Selection")
    pdb.gimp_selection_none(image)
    
    #ensure image is in RGB
    if(pdb.gimp_image_base_type(image) != RGB) :
        pdb.gimp_image_convert_rgb(image)
            
    
    pdb.gimp_context_set_interpolation(INTERPOLATION_NONE);
    pdb.gimp_context_set_transform_resize(TRANSFORM_RESIZE_ADJUST);
        
    pdb.gimp_message("Enlarging Horizontally...")  #Enlarge Image Horizontally   
    layerTemp1 = pdb.gimp_layer_copy(inLayer, TRUE)
    pdb.gimp_image_add_layer(image, layerTemp1, -1)
    pdb.gimp_layer_resize(layerTemp1, 1, height, 0,0)
    
    layerTemp2 = pdb.gimp_layer_copy(inLayer, TRUE)
    pdb.gimp_image_add_layer(image, layerTemp2, -1)
    pdb.gimp_layer_resize(layerTemp2, 1, height, (width -1), 0)
    
    pdb.gimp_image_resize(image, (width + inGridSize + inGridSize),height, inGridSize, 0)
    layerTemp1 = pdb.gimp_item_transform_scale(layerTemp1, 0, 0, inGridSize, height)
    layerTemp2 = pdb.gimp_item_transform_scale(layerTemp2,(inGridSize + width), 0, (width + (inGridSize * 2)), height)
    
    
    layerTemp1 = pdb.gimp_image_merge_down(image, layerTemp2, EXPAND_AS_NECESSARY)
    inLayer = pdb.gimp_image_merge_down(image, layerTemp1, EXPAND_AS_NECESSARY)
        
    width = pdb.gimp_image_width(image);
        
    pdb.gimp_message("Enlarging Vertically...")    #Enlarge Image Vertically
    layerTemp1 = pdb.gimp_layer_copy(inLayer, TRUE)  
    pdb.gimp_image_add_layer(image, layerTemp1, -1)
    pdb.gimp_layer_resize(layerTemp1, width, 1, 0,0)
    
    layerTemp2 = pdb.gimp_layer_copy(inLayer, TRUE)
    pdb.gimp_image_add_layer(image, layerTemp2, -1)
    pdb.gimp_layer_resize(layerTemp2, width, 1 , 0, (height -1))
    
    pdb.gimp_image_resize(image, width, (height + inGridSize + inGridSize),0 , inGridSize)
    layerTemp1 = pdb.gimp_item_transform_scale(layerTemp1, 0, 0, width, inGridSize)
    layerTemp2 = pdb.gimp_item_transform_scale(layerTemp2, 0, (inGridSize + height), width, (height + (inGridSize * 2)))
    
    
    layerTemp1 = pdb.gimp_image_merge_down(image, layerTemp2, EXPAND_AS_NECESSARY)
    inLayer = pdb.gimp_image_merge_down(image, layerTemp1, EXPAND_AS_NECESSARY)
    
    height = pdb.gimp_image_height(image)
    
    origLayer = inLayer;
    
    pdb.gimp_displays_flush()

    #Select by Color 
    pdb.gimp_message("Selecting By Color...")    
    pdb.gimp_context_set_antialias(TRUE)
    pdb.gimp_context_set_feather(FALSE)
    pdb.gimp_context_set_feather_radius(0,0)
    pdb.gimp_context_set_sample_merged(FALSE)
    pdb.gimp_image_select_color(image, CHANNEL_OP_REPLACE, inLayer, inSelectColor)
    
    if(inSelectWhat == 1) :
        pdb.gimp_message("Inverting Selection...")
        pdb.gimp_selection_invert(image)
    #end if
    
    selInside = pdb.gimp_selection_save(image)
    pdb.gimp_item_set_name(selInside, "Dungeon Mask")
    pdb.gimp_selection_none(image)
    
    
    
    #Building Floor
    if(inFloorFill != 0) : 
        pdb.gimp_message("Creating Floor Layer")
        layerFloor = pdb.gimp_layer_new_from_drawable(inLayer, image);
        pdb.gimp_item_set_name(layerFloor, varLayerName + " floor")
        pdb.gimp_image_add_layer(image, layerFloor, -1)
        pdb.gimp_layer_add_alpha(layerFloor)    
        
        pdb.gimp_displays_flush()
        
        if(inFloorFill == 1) :
            randSeed = random.randint(1, 2147483690)
            pdb.gimp_message("Filling Floor with Rendered Noise " + str(randSeed))        
            pdb.plug_in_plasma(image, layerFloor, randSeed, 7)
            pdb.gimp_drawable_desaturate(layerFloor, DESATURATE_LIGHTNESS)
            pdb.gimp_drawable_levels(layerFloor, HISTOGRAM_VALUE,0.0,1.0,TRUE,1,0.43,0.58,TRUE)
            pdb.gimp_message("Bumping the Floor")        
            pdb.plug_in_bump_map(image, layerFloor, layerFloor, inLightAngle, 50, 30, 0,0,0,0,TRUE,TRUE,0)
        elif(inFloorFill == 2) :
            pdb.gimp_message("Filling Floor pattern ")
            pdb.gimp_context_set_pattern(inFloorPattern)
            pdb.gimp_drawable_fill(layerFloor, FILL_PATTERN)
        #end if
        
        pdb.gimp_displays_flush()
        
        if(inNoiseOverlay == 1) :
            randSeed = random.randint(1, 2147483690)
            pdb.gimp_message("Building Floor Overlay " + str(randSeed))
            layerFloorOverlay = pdb.gimp_layer_new_from_drawable(inLayer, image)
            pdb.gimp_item_set_name(layerFloorOverlay, varLayerName + " floor noise")
            pdb.gimp_image_add_layer(image, layerFloorOverlay, -1)
            pdb.gimp_layer_add_alpha(layerFloorOverlay)
            pdb.plug_in_solid_noise(image, layerFloorOverlay, FALSE, FALSE, randSeed, 4, 4, 4)
            pdb.gimp_layer_set_mode(layerFloorOverlay, LAYER_MODE_OVERLAY_LEGACY)
        #end if
        
        pdb.gimp_displays_flush()
        
    else :
        pdb.gimp_message("No Floor")
    #end floor if
    
    if(inGrid == 1) :
        pdb.gimp_message("Building Grid")
        layerGrid = pdb.gimp_layer_new_from_drawable(inLayer, image)
        pdb.gimp_item_set_name(layerGrid, varLayerName + " grid")
        pdb.gimp_image_add_layer(image, layerGrid, -1)
        pdb.gimp_layer_add_alpha(layerGrid)
        pdb.gimp_selection_all(image)
        pdb.gimp_edit_clear(layerGrid)
        pdb.gimp_selection_none(image)
        gw = (inGridSize/50)        
        pdb.plug_in_grid(image, layerGrid, max([1,gw]) ,inGridSize, 0, (0,0,0), 255, max([1,gw]),inGridSize, 0, (0,0,0), 255, 0,0,0, (0,0,0), 255) 
        #pdb.script_fu_grid_system(RUN_NONINTERACTIVE, image, layerGrid, inGridSize)
        
        pdb.gimp_displays_flush()
        
        #blured copy for bump map
        pdb.gimp_message("Building Blurred Copy")
        layerTemp1 = pdb.gimp_layer_new_from_drawable(layerGrid, image)
        pdb.gimp_item_set_name(layerTemp1, varLayerName + " grid temp")
        pdb.gimp_image_add_layer(image, layerTemp1, -1)
        pdb.gimp_context_set_background((255,255,255))
        pdb.gimp_layer_flatten(layerTemp1)
        pdb.plug_in_gauss(image, layerTemp1, (inGridSize/10), (inGridSize/10), 1)
        
        pdb.gimp_displays_flush()
        
        #bumping grid
        pdb.gimp_message("Bumping Grid")
        layerGridBump = pdb.gimp_layer_new_from_drawable(layerGrid, image)
        pdb.gimp_item_set_name(layerGridBump, varLayerName + " grid bump")
        pdb.gimp_image_add_layer(image, layerGridBump, -1)
        pdb.gimp_context_set_foreground((127,127,127))
        pdb.gimp_drawable_fill(layerGridBump, FOREGROUND_FILL)
        pdb.plug_in_bump_map(image, layerGridBump, layerTemp1, inLightAngle, 10,30,0,0,0,0,TRUE, FALSE, 1)
        pdb.gimp_layer_set_mode(layerGridBump, LAYER_MODE_OVERLAY)
        
        pdb.gimp_displays_flush()
        
        #removing blurred layer
        pdb.gimp_message("Removing Blurred Grid Layer")
        pdb.gimp_image_remove_layer(image,layerTemp1)
        
        #set Grid Color as desired
        pdb.plug_in_grid(image, layerGrid, max([1, gw]) ,inGridSize, 0, inGridColor, 255, max([1, gw]) ,inGridSize, 0, inGridColor, 255, 0,0,0, inGridColor, 255) 
        
        pdb.gimp_displays_flush()
    else :
        pdb.gimp_message("No Grid")
    #end grid if
    
    #create wall shadows
    pdb.gimp_message("Building Wall Shadows")
    layerWallShadows = pdb.gimp_layer_new_from_drawable(inLayer, image)
    pdb.gimp_item_set_name(layerWallShadows, varLayerName + " shadows")
    pdb.gimp_image_add_layer(image, layerWallShadows, -1)
    pdb.gimp_layer_add_alpha(layerWallShadows)
    pdb.gimp_context_set_foreground((0,0,0))
    pdb.gimp_drawable_fill(layerWallShadows, FILL_FOREGROUND)
    pdb.gimp_context_set_foreground((255,255,255))
    pdb.gimp_selection_load(selInside)
    shrink = inGridSize/8
    pdb.gimp_message("    Shrinking Selection by " + str(shrink))
    pdb.gimp_selection_shrink(image, shrink)
    if(inWallStyle == 1):  #adjust if stroked
        if(inStrokePos == 0):
            pdb.gimp_selection_shrink(image, (inStrokeSize/2))  #centered
        elif(inStrokePos == 2):
            pdb.gimp_selection_shrink(image, inStrokeSize)      #inside
    #end if stroked
    
    pdb.gimp_edit_bucket_fill(layerWallShadows, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 0, FALSE, 0, 0)
    pdb.gimp_selection_none(image)
    pdb.plug_in_gauss(image, layerWallShadows, (inGridSize/3), (inGridSize/3), 1)
    pdb.gimp_selection_load(selInside)
    pdb.gimp_selection_invert(image)
    pdb.gimp_edit_bucket_fill(layerWallShadows, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 0, FALSE, 0, 0)
    pdb.gimp_selection_none(image)
    pdb.plug_in_colortoalpha(image, layerWallShadows, (255,255,255))
    #pdb.gimp_layer_set_mode(layerWallShadows, LAYER_MODE_MULTIPLY)
    pdb.gimp_layer_set_opacity(layerWallShadows, 80)
    
    pdb.gimp_displays_flush()
    
    #create Filled Wall Layer
    pdb.gimp_message("Building Filled Wall Layer")
    layerWall = pdb.gimp_layer_new_from_drawable(inLayer, image)
    pdb.gimp_item_set_name(layerWall, varLayerName + " wall")
    pdb.gimp_image_add_layer(image, layerWall, -1)
    pdb.gimp_layer_add_alpha(layerWall)
    
    if(inWallMode == 0) :
        pdb.gimp_message("Filling Wall Rendered Noise")
        randSeed = random.randint(1, 2147483690)
        pdb.plug_in_plasma(image, layerWall, randSeed, 7);
        pdb.gimp_drawable_desaturate(layerWall, DESATURATE_LIGHTNESS)
        
        pdb.gimp_drawable_levels(layerWall, HISTOGRAM_VALUE,0.0,1.0,FALSE,1,0.15,0.35,FALSE)
        
        pdb.plug_in_bump_map(image, layerWall, layerWall, inLightAngle, 80,30,0,0,0,0,FALSE, FALSE, 0)
               
        
        
        #cracks
        pdb.gimp_message("Building Wall Cracks")
        layerTemp1 = pdb.gimp_layer_new_from_drawable(inLayer, image)
        pdb.gimp_image_add_layer(image, layerTemp1, -1)
        randSeed = random.randint(1, 2147483690)
        pdb.plug_in_solid_noise(image, layerTemp1, FALSE, TRUE, randSeed, 15, 15, 15)
        
        #layerTemp3 = pdb.gimp_layer_new_from_drawable(layerTemp1, image)
        #pdb.gimp_image_add_layer(image, layerTemp3, 0)
        #pdb.gimp_item_set_name(layerTemp3, varLayerName + " wall temp 1-1")
        #pdb.gimp_drawable_set_visible(layerTemp3, FALSE)
        
        layerTemp2 = pdb.gimp_layer_new_from_drawable(inLayer, image)  #difference clouds
        pdb.gimp_image_add_layer(image, layerTemp2, -1)        
        randSeed = random.randint(1, 2147483690)
        pdb.plug_in_solid_noise(image, layerTemp2, FALSE, TRUE, randSeed, 15, 15, 15)
        pdb.gimp_layer_set_mode(layerTemp2, LAYER_MODE_DIFFERENCE)
        
        #layerTemp3 = pdb.gimp_layer_new_from_drawable(layerTemp2, image)
        #pdb.gimp_image_add_layer(image, layerTemp3, 0)
        #pdb.gimp_item_set_name(layerTemp3, varLayerName + " wall temp 2-1")
        #pdb.gimp_drawable_set_visible(layerTemp3, FALSE)
        
        layerTemp1 = pdb.gimp_image_merge_down(image,layerTemp2,EXPAND_AS_NECESSARY)
        
        layerTemp2 = pdb.gimp_layer_new_from_drawable(inLayer, image)  #difference clouds
        pdb.gimp_image_add_layer(image, layerTemp2, -1)
        randSeed = random.randint(1, 2147483690)
        pdb.plug_in_solid_noise(image, layerTemp2, FALSE, TRUE, randSeed, 15, 15, 15)
        pdb.gimp_layer_set_mode(layerTemp2, LAYER_MODE_DIFFERENCE)
        
        #layerTemp3 = pdb.gimp_layer_new_from_drawable(layerTemp2, image)
        #pdb.gimp_image_add_layer(image, layerTemp3, 0)
        #pdb.gimp_drawable_set_visible(layerTemp3, FALSE)
        #pdb.gimp_item_set_name(layerTemp3, varLayerName + " wall temp 2-2")
        
        layerTemp1 = pdb.gimp_image_merge_down(image,layerTemp2,EXPAND_AS_NECESSARY)
        pdb.gimp_item_set_name(layerTemp1, varLayerName + " wall temp")        
        pdb.gimp_drawable_levels(layerTemp1, HISTOGRAM_VALUE,0.0,0.17,FALSE,0.1,0,1,FALSE)       
        
        #layerTemp3 = pdb.gimp_layer_new_from_drawable(layerTemp1, image)
        #pdb.gimp_image_add_layer(image, layerTemp3, 0)
        #pdb.gimp_drawable_set_visible(layerTemp3, FALSE)
        #pdb.gimp_item_set_name(layerTemp3, varLayerName + " wall temp 1-2")
        
        #bumpmap cracks
        pdb.plug_in_bump_map(image, layerWall, layerTemp1, inLightAngle, 40,2,0,0,0,0, TRUE, FALSE, 1) 
        pdb.gimp_image_remove_layer(image, layerTemp1)
    else :
        pdb.gimp_context_set_pattern(inWallPattern)
        pdb.gimp_drawable_fill(layerWall, FILL_PATTERN)
     
    pdb.gimp_selection_load(selInside)
    pdb.gimp_selection_invert(image)
    maskWall = pdb.gimp_layer_create_mask(layerWall, ADD_MASK_SELECTION)
    pdb.gimp_layer_add_mask(layerWall, maskWall)
    pdb.gimp_selection_none(image)
    
    pdb.gimp_displays_flush()
    
    #wall styling
    pdb.gimp_message("Wall Styling: " + str(inWallStyle))
    if(inWallStyle == 0) :  #beveled
        pdb.gimp_message("Beveled Wall Style")
        layerTemp1 = pdb.gimp_layer_new_from_drawable(inLayer, image)
        pdb.gimp_image_add_layer(image, layerTemp1, -1)
        pdb.gimp_context_set_foreground((255,255,255))
        pdb.gimp_drawable_fill(layerTemp1, FILL_FOREGROUND)
        pdb.gimp_selection_load(selInside)
        pdb.gimp_context_set_foreground((0,0,0))
        pdb.gimp_edit_bucket_fill(layerTemp1, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 0, FALSE,0,0)
        pdb.gimp_selection_none(image)
        pdb.plug_in_gauss(image, layerTemp1, (inGridSize/8), (inGridSize/8), 1)
        
        #create the wall bevel
        layerWallStyle =  pdb.gimp_layer_new_from_drawable(inLayer, image)
        pdb.gimp_item_set_name(layerWallStyle, varLayerName + " wall style")
        pdb.gimp_image_add_layer(image, layerWallStyle, -1)
        pdb.gimp_layer_add_alpha(layerWallStyle)
        pdb.gimp_context_set_foreground((127,127,127))
        pdb.gimp_drawable_fill(layerWallStyle, FILL_FOREGROUND)
        pdb.plug_in_bump_map(image, layerWallStyle, layerTemp1, inLightAngle, 10,10,0,0,0,0, TRUE, FALSE, 0) 
        pdb.gimp_layer_set_mode(layerWallStyle, LAYER_MODE_OVERLAY)
        
        pdb.gimp_image_remove_layer(image, layerTemp1)  #remove the temp layer
        
        pdb.gimp_selection_load(selInside)
        pdb.gimp_selection_invert(image)
        maskWall = pdb.gimp_layer_create_mask(layerWallStyle, ADD_MASK_SELECTION)
        pdb.gimp_layer_add_mask(layerWallStyle, maskWall)
        pdb.gimp_selection_none(image)
        
    elif (inWallStyle == 1) :  #stroked
        pdb.gimp_message("Stroked Wall Style")
        layerWallStyle =  pdb.gimp_layer_new_from_drawable(inLayer, image)
        pdb.gimp_item_set_name(layerWallStyle, varLayerName + " wall style")
        pdb.gimp_image_add_layer(image, layerWallStyle, -1)
        pdb.gimp_layer_add_alpha(layerWallStyle)
        pdb.gimp_edit_clear(layerWallStyle)
        
        pdb.gimp_context_set_foreground(inStrokeColor)
        
        pdb.gimp_selection_load(selInside)
        
        if(inStrokePos == 0) : #centered
            pdb.gimp_selection_grow(image, (inStrokeSize/2))
            pdb.gimp_edit_fill(layerWallStyle, FILL_FOREGROUND)
            pdb.gimp_selection_load(selInside)
            pdb.gimp_selection_shrink(image, (inStrokeSize/2))
            pdb.gimp_edit_clear(layerWallStyle)
        elif(inStrokePos == 1) : #outside
            pdb.gimp_selection_grow(image, inStrokeSize)
            pdb.gimp_edit_fill(layerWallStyle, FILL_FOREGROUND)
            pdb.gimp_selection_load(selInside)
            pdb.gimp_edit_clear(layerWallStyle)
        else :  #inside
            pdb.gimp_edit_fill(layerWallStyle, FILL_FOREGROUND)
            pdb.gimp_selection_shrink(image, inStrokeSize)
            pdb.gimp_edit_clear(layerWallStyle)
            
         
        
        pdb.gimp_selection_none(image)
        
        if(inStrokeStyle >= 2) :  #pattern filled
            pdb.gimp_selection_layer_alpha(layerWallStyle)
            pdb.gimp_context_set_pattern(inStrokePattern)
            pdb.gimp_edit_fill(layerWallStyle, FILL_PATTERN)
            pdb.gimp_selection_none(image)
            
        if(inStrokeStyle == 1 or inStrokeStyle == 3) :
            layerTemp1 = pdb.gimp_layer_new_from_drawable(inLayer, image)
            pdb.gimp_image_add_layer(image, layerTemp1, -1)
            pdb.gimp_context_set_foreground((127,127,127))
            pdb.gimp_edit_fill(layerTemp1, FILL_FOREGROUND)
            
            layerTemp2 = pdb.gimp_layer_new_from_drawable(inLayer, image)
            pdb.gimp_image_add_layer(image, layerTemp2, -1)
            pdb.gimp_context_set_foreground((0,0,0))
            pdb.gimp_edit_fill(layerTemp2, FILL_FOREGROUND)
            pdb.gimp_selection_layer_alpha(layerWallStyle)
            pdb.gimp_context_set_foreground((255,255,255))
            pdb.gimp_edit_fill(layerTemp2, FILL_FOREGROUND)
            pdb.gimp_selection_none(image)
            pdb.plug_in_gauss(image, layerTemp2, (inStrokeSize/2), (inStrokeSize/2), 1)
            
            pdb.plug_in_bump_map(image, layerTemp1, layerTemp2, inLightAngle, 10,10,0,0,0,0, TRUE, FALSE, 0) 
            pdb.gimp_layer_set_mode(layerTemp1, LAYER_MODE_OVERLAY)
            
            pdb.gimp_image_remove_layer(image, layerTemp2)  #remove the temp layer
            layerWallStyle = pdb.gimp_image_merge_down(image, layerTemp1, EXPAND_AS_NECESSARY)
        
    pdb.gimp_displays_flush()  
    
    #wall fade
    if(inWallFade == 1 or inWallFade == 2) :  #black or white
        layerWallFade = pdb.gimp_layer_new_from_drawable(inLayer, image)
        pdb.gimp_item_set_name(layerWallFade, varLayerName + " wall fade")
        pdb.gimp_image_add_layer(image, layerWallFade, -1)
        pdb.gimp_layer_add_alpha(layerWallFade)
        
        if(inWallFade == 1) : 
            pdb.gimp_context_set_foreground((0,0,0))
        else :
            pdb.gimp_context_set_foreground((255,255,255))
            
        pdb.gimp_drawable_fill(layerWallFade, FILL_FOREGROUND)
        pdb.gimp_selection_load(selInside)
        pdb.gimp_selection_grow(image, (inGridSize/2))
        pdb.gimp_selection_feather(image, (inGridSize/1.5))
        pdb.gimp_edit_clear(layerWallFade)
        pdb.gimp_selection_none(image)
    elif(inWallFade == 3) :  #Transparent
        pdb.gimp_context_set_foreground((0,0,0))
        pdb.gimp_selection_load(selInside)
        pdb.gimp_selection_grow(image, (inGridSize/2))
        pdb.gimp_selection_feather(image, (inGridSize/1.1))
        pdb.gimp_selection_invert(image)
        if(layerFloor != 0) : pdb.gimp_edit_clear(layerFloor)
        if(layerFloorOverlay != 0) : pdb.gimp_edit_clear(layerFloorOverlay)
        if(layerWall != 0) : pdb.gimp_edit_clear(layerWall)
        if(layerGrid != 0) : pdb.gimp_edit_clear(layerGrid)
        if(layerGridBump != 0) : pdb.gimp_edit_clear(layerGridBump)
        if(layerWallStyle != 0) : pdb.gimp_edit_clear(layerWallStyle)
        if(layerWallShadows != 0) : pdb.gimp_edit_clear(layerWallShadows)
        pdb.gimp_selection_none(image)
        
    pdb.gimp_displays_flush()

    pdb.gimp_image_remove_channel(image, selInside)
    
    pdb.gimp_image_crop(image, (width - (2* inGridSize)), (height - (2*inGridSize)), inGridSize, inGridSize)
    if(layerFloor == 0) :
        pdb.gimp_image_set_active_layer(image, origLayer)
    else:
        pdb.gimp_image_set_active_layer(image, layerFloor)
        
    pdb.gimp_drawable_set_visible(origLayer, FALSE)
    
    pdb.gimp_image_undo_group_end(image)
    pdb.gimp_displays_flush()
    pdb.gimp_progress_end()
    pdb.gimp_context_pop()
    
    


register(
    "python-fu-dungeon-map-maker",
    "Dungeon Map Maker Ver.1.8",
    "Based on original Dungeon Map Maker from Rob Antonishen on Cartographer's Guild",
    "Keith Reinig", "Keith Reinig", "2020",
    "Dungeon Map Maker",
    "*",
    [
        (PF_IMAGE, "image", "takes current image", None),
        (PF_DRAWABLE, "inLayer", "Input layer", None),
        (PF_SPINNER, "inGridSize",  "Grid", 50, (0, 200, 1)),
        (PF_COLOR, "inSelectColor", "Template Color", (255, 255, 255)),
        (PF_SPINNER, "inSelectThresh",  "Color Selection Threshold", 10, (0, 255, 1)),
        (PF_OPTION, "inSelectWhat", "Template Color Identifies", 0, ("Floors", "Walls")),        
        (PF_SPINNER, "inLightAngle",  "Lighting Azimuth", 135, (0, 360, 1)),
        (PF_OPTION, "inFloorFill", "Fill Floor With", 0, ("No Floor", "Rendered Noise", "Pattern - Select Below")),
        (PF_PATTERN, "inFloorPattern", "Floor Pattern", "Dried mud"),
        (PF_TOGGLE, "inNoiseOverlay", "Add Noise Overlay", 1),
        (PF_TOGGLE, "inGrid", "Draw Grid", 1),
        (PF_COLOR, "inGridColor", "Grid Color", (0, 0, 0)),
        (PF_OPTION, "inWallMode", "Fill Wall With", 0, ("Rendered Noise", "Pattern - Select Below")),
        (PF_PATTERN, "inWallPattern", "Wall Pattern", "Crack"),
        (PF_OPTION, "inWallStyle", "Wall Style", 0, ("Bevel", "Stroked", "None")),
        (PF_OPTION, "inWallFade", "Wall Fade", 0, ("None", "Black", "White", "Transparent")),
        (PF_OPTION, "inStrokeStyle", "Stroke Style", 0, ("Color", "Color-Beveled", "Pattern", "Pattern-Beveled")),        
        (PF_SPINNER, "inStrokeSize",  "Stroke Size", 5, (1, 50, 1)),
        (PF_OPTION, "inStrokePos", "Stroke Position", 0, ("Centered", "Outside", "Inside")),
        (PF_COLOR, "inStrokeColor", "Stroke Color", (128, 128, 128)),
        (PF_PATTERN, "inStrokePattern", "Stroke Pattern", "Fibers"),
        
    ],
    [],
    dungeon_map_maker, menu="<Image>/Filters/Render")
    
main()